package org.rcisoft.business.overview.service.impl;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.business.overview.dao.BusProjectAreaRepository;
import org.rcisoft.business.overview.dao.BusProjectAreaStatRepository;
import org.rcisoft.business.overview.dao.EnergyCountMRepository;
import org.rcisoft.business.overview.dao.EnergyPriceRepository;
import org.rcisoft.business.overview.entity.BusProjectArea;
import org.rcisoft.business.overview.entity.BusProjectAreaStat;
import org.rcisoft.business.overview.entity.EnergyCountM;
import org.rcisoft.business.overview.entity.EnergyPrice;
import org.rcisoft.business.overview.service.OverViewService;
import org.rcisoft.business.overview.vo.EnergyCountMVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * Created by JiChao on 2018/4/11.
 */
@Service
@Slf4j
public class OverViewServiceImpl implements OverViewService {

    @Autowired
    private EnergyPriceRepository overViewRepository;
    @Autowired
    private EnergyCountMRepository energyCountMRepository;
    @Autowired
    private BusProjectAreaRepository busProjectAreaRepository;
    @Autowired
    private BusProjectAreaStatRepository busProjectAreaStatRepository;

    private Calendar getBeforeDay(Calendar cl, int time){
        int day = cl.get(Calendar.DATE);
        cl.set(Calendar.DATE, day + time);
        return cl;
    }

    private Map<String, String> getTime(int time){
        Calendar now = Calendar.getInstance();
        if(time != 0) now = this.getBeforeDay(now, time);
        Map<String, String> result = new HashMap<String, String>();
        result.put("year", String.valueOf(now.get(Calendar.YEAR)));
        result.put("month", String.valueOf(now.get(Calendar.MONTH) + 1));
        result.put("day", String.valueOf(now.get(Calendar.DAY_OF_MONTH)));
        result.put("hour", String.valueOf(now.get(Calendar.HOUR_OF_DAY)));
        return result;
    }

    private Map<String, String> getTime(String year, String month, String day, int yearVar, int monthVar, int dayVar){
        Calendar calendar = Calendar.getInstance();
        if(year != null){
            calendar.set(Calendar.YEAR, Integer.parseInt(year) + yearVar);
        }
        if(month != null){
            calendar.set(Calendar.MONTH, Integer.parseInt(month) - 1 + monthVar);
        }
        if(day != null){
            calendar.set(Calendar.DATE, Integer.parseInt(day) + dayVar);
        }
        Map<String, String> result = new HashMap<String, String>();
        result.put("year", String.valueOf(calendar.get(Calendar.YEAR)));
        result.put("month", String.valueOf(calendar.get(Calendar.MONTH) + 1));
        result.put("day", String.valueOf(calendar.get(Calendar.DAY_OF_MONTH)));
        return result;
    }

    public static void main(String[] args){
        OverViewServiceImpl o = new OverViewServiceImpl();
        Map<String, String> time = o.getTime("2016", "2", "29", 2, 0, 0);
        System.out.println(time.get("year") + "-" + time.get("month") + "-" + time.get("day"));
    }

    private String getType(String type) {
        switch (type) {
            case "1": //水
                type = "water";
                break;
            case "2": //电
                type = "elec";
                break;
            case "3": //气
                type = "gas";
                break;
            default:
                break;
        }
        return type;
    }

    @Override
    public Map<String, Object> getPrice(String proId) {
        Map<String, Object> map = new HashMap<String, Object>();
        List<EnergyPrice> list = overViewRepository.getPrice(proId);
        Iterator<EnergyPrice> it = list.iterator();
        while (it.hasNext()) {
            EnergyPrice energyPrice = it.next();
            String type = this.getType(energyPrice.getPTp());
            if(StringUtils.isNotEmpty(type)) map.put(type, energyPrice);
        }
        return map;
    }

    @Override
    public Map<String, Object> countAmountAndPrice(String proId) {
        Map<String, String> cal = this.getTime(0);
        EnergyCountM m = new EnergyCountM();
        m.setProId(proId);
        m.setYear(cal.get("year"));
        //统计日
        m.setDay(cal.get("day"));
        EnergyCountM dayCount = energyCountMRepository.countAmountAndPrice(m);
        //统计月
        m.setDay(null);
        m.setMon(cal.get("month"));
        EnergyCountM monthCount = energyCountMRepository.countAmountAndPrice(m);
        //合并返回值
        Map<String, Object> result = new HashMap<String, Object>();
        result.put("day", dayCount);
        result.put("mon", monthCount);
        return result;
    }

    @Override
    public List<EnergyCountMVo> countDailyPrice(String proId, Integer type, Integer date) {
        Map<String, String> cal = this.getTime(date);
        EnergyCountMVo e = new EnergyCountMVo();
        e.setProId(proId);
        e.setType(type);
        e.setYear(cal.get("year"));
        e.setMon(cal.get("month"));
        e.setDay(cal.get("day"));
        return energyCountMRepository.countDailyPrice(e);
    }

    @Override
    public List<EnergyCountMVo> countEnergySplit(String proId, Integer type, Integer date) {
        Map<String, String> cal = this.getTime(date);
        EnergyCountMVo e = new EnergyCountMVo();
        e.setProId(proId);
        e.setType(type);
        e.setYear(cal.get("year"));
        e.setMon(cal.get("month"));
        e.setDay(cal.get("day"));
        return energyCountMRepository.countEnergySplit(e);
    }

    @Override
    public Integer getPriceRank(String proId, Integer type, Integer date) {
        Map<String, String> cal = this.getTime(date);
        String day = cal.get("day");
        EnergyCountMVo e = new EnergyCountMVo();
        e.setProId(proId);
        e.setType(type);
        e.setYear(cal.get("year"));
        e.setMon(cal.get("month"));
        e.setDay(day);
        if(date != 0) e.setHour(cal.get("hour"));
        List<EnergyCountMVo> list = energyCountMRepository.getPriceRank(e);
        for (int i = 0; i < list.size(); i++) {
            EnergyCountMVo energyCountMVo = list.get(i);
            String eDay = energyCountMVo.getDay();
            if(StringUtils.equals(day, eDay))
                return i + 1;
        }
        return 1;
    }

    @Override
    public Map<String, Object> getStandard(String proId) {
        BusProjectArea b = new BusProjectArea();
        b.setProId(proId);
        List<BusProjectArea> list = busProjectAreaRepository.queryBusProjectAreas(b);
        //返回值
        Map<String, Object> result = new HashMap<String, Object>();
        Iterator<BusProjectArea> it = list.iterator();
        while (it.hasNext()) {
            BusProjectArea busProjectArea = it.next();
            String type = this.getType(busProjectArea.getType());
            if(StringUtils.isNotEmpty(type)) result.put(type, busProjectArea);
        }
        return result;
    }

    @Override
    public Map<String, Object> getUnitArea(String proId) {
        String thisY = getTime(0).get("year");
        int i = Integer.parseInt(thisY);
        String lastY = String.valueOf(i - 1);
        String beforeY = String.valueOf(i - 2);
        BusProjectAreaStat b = new BusProjectAreaStat();
        b.setProId(proId);
        //查询当年
        b.setYear(thisY);
        BusProjectAreaStat thisBus = busProjectAreaStatRepository.queryBusProjectAreaStats(b);
        //查询去年
        b.setYear(lastY);
        BusProjectAreaStat lastBus = busProjectAreaStatRepository.queryBusProjectAreaStats(b);
        //查询前年
        b.setYear(beforeY);
        BusProjectAreaStat beforeBus = busProjectAreaStatRepository.queryBusProjectAreaStats(b);
        //返回结果
        Map<String, Object> result = new HashMap<String, Object>();
        if(beforeBus == null) {
            beforeBus = new BusProjectAreaStat(beforeY, 0f, 0f, 0f);
        }
        if(lastBus == null) {
            lastBus = new BusProjectAreaStat(beforeY, 0f, 0f, 0f);
        }
        if(thisBus == null) {
            thisBus = lastBus;
        }
        result.put("thisY", thisBus);
        result.put("lastY", lastBus);
        result.put("beforeY", beforeBus);
        return result;
    }

    @Override
    public Map<String, Object> getEnergyCompare(String proId, String year, String month, Integer type, Integer cType) {
        //返回值
        Map<String, Object> result = new HashMap<String, Object>();
        //添加当前年月
        result.put("year", year);
        result.put("month", month);
        EnergyCountMVo e = new EnergyCountMVo();
        e.setProId(proId);
        e.setType(type);
        e.setYear(year);
        e.setMon(month);
        //查找现在的数据
        List<EnergyCountMVo> nowCompare = energyCountMRepository.getEnergyCompare(e);
        //现在数据最后返回的对象
        List<Float> nowReturn = this.getFloatList(nowCompare);
        //查找需要比较的数据
        Map<String, String> time = null;
        //同比，环比，确定时间
        if(cType == 1){//同比
            time = this.getTime(year, month, null, -1, 0, 0);
        }else{//环比
            time = this.getTime(year, month, null, 0, -1, 0);
        }
        //新的时间
        String c_year = time.get("year");
        String c_month = time.get("month");
        e.setYear(c_year);
        e.setMon(c_month);
        //添加当前年月
        result.put("cYear", c_year);
        result.put("cMonth", c_month);
        List<EnergyCountMVo> lastCompare = energyCountMRepository.getEnergyCompare(e);
        List<Float> lastReturn = this.getFloatList(lastCompare);
        //差值
        List<Float> diffReturn = new ArrayList<Float>();
        for (int i = 0; i < nowReturn.size(); i++) {
            diffReturn.add(nowReturn.get(i) - lastReturn.get(i));
        }
        //数据值
        result.put("nData", nowReturn);
        result.put("cData", lastReturn);
        result.put("dData", diffReturn);
        return result;
    }

    private List<Float> getFloatList(List<EnergyCountMVo> list){
        List<Float> resultList = new ArrayList<Float>();
        if(list != null && list.size() > 0){
            int size = list.size();
            Integer start = Integer.parseInt(list.get(0).getDay());
            Integer end = Integer.parseInt(list.get(size - 1).getDay());
            for (int i = 1; i < start; i++) {
                resultList.add(0f);
            }
            for (int i = 0; i < size; i++) {
                resultList.add(list.get(i).getEnergy());
            }
            for (int i = end; i < 31; i++) {
                resultList.add(0f);
            }
        }else{
            for (int i = 0; i < 31; i++) {
                resultList.add(0f);
            }
        }
        return resultList;
    }
}
